@@ -2,6 +2,10 @@
 //! SVD objects.
 //! This module defines components of an SVD along with parse and encode implementations
 
+/// Common things for structures which can be collected in arrays
+pub mod array;
+pub use array::MaybeArray;
+
 /// Endian objects
 pub mod endian;
 pub use self::endian::Endian;
@@ -40,15 +44,7 @@ pub use self::enumeratedvalues::{EnumeratedValues, EnumeratedValuesBuilder};
 
 /// Field objects
 pub mod field;
-pub use self::field::Field;
-
-/// Field Info objects
-pub mod fieldinfo;
-pub use self::fieldinfo::{FieldInfo, FieldInfoBuilder};
-
-/// Register Info objects
-pub mod registerinfo;
-pub use self::registerinfo::{RegisterInfo, RegisterInfoBuilder};
+pub use self::field::{Field, FieldInfo, FieldInfoBuilder};
 
 /// Register Properties objects
 pub mod registerproperties;
@@ -60,15 +56,11 @@ pub use self::addressblock::{AddressBlock, AddressBlockUsage};
 
 /// Cluster objects
 pub mod cluster;
-pub use self::cluster::Cluster;
-
-/// Cluster Info objects
-pub mod clusterinfo;
-pub use self::clusterinfo::{ClusterInfo, ClusterInfoBuilder};
+pub use self::cluster::{Cluster, ClusterInfo, ClusterInfoBuilder};
 
 /// Register objects
 pub mod register;
-pub use self::register::Register;
+pub use self::register::{Register, RegisterInfo, RegisterInfoBuilder};
 
 /// Register Cluster objects
 pub mod registercluster;
@@ -76,11 +68,11 @@ pub use self::registercluster::RegisterCluster;
 
 /// Dimelement objects
 pub mod dimelement;
-pub use self::dimelement::{DimElement, DimElementBuilder};
+pub use self::dimelement::{DimArrayIndex, DimElement, DimElementBuilder};
 
 /// Peripheral objects
 pub mod peripheral;
-pub use self::peripheral::{Peripheral, PeripheralBuilder};
+pub use self::peripheral::{Peripheral, PeripheralInfo, PeripheralInfoBuilder};
 
 /// Device objects
 pub mod device;
@@ -90,6 +82,14 @@ pub use self::device::{Device, DeviceBuilder};
 pub mod modifiedwritevalues;
 pub use self::modifiedwritevalues::ModifiedWriteValues;
 
+/// Read Action objects
+pub mod readaction;
+pub use self::readaction::ReadAction;
+
+/// Protection objects
+pub mod protection;
+pub use self::protection::Protection;
+
 /// Level of validation
 #[derive(Clone, Copy, Debug, PartialEq)]
 pub enum ValidateLevel {
@@ -147,13 +147,13 @@ pub enum SvdError {
     Peripheral(#[from] peripheral::Error),
     /// Cluster error
     #[error("`Cluster error: {0}")]
-    Cluster(#[from] clusterinfo::Error),
+    Cluster(#[from] cluster::Error),
     /// Register error
     #[error("`Register error: {0}")]
-    Register(#[from] registerinfo::Error),
+    Register(#[from] register::Error),
     /// Field error
     #[error("`Field error: {0}")]
-    Field(#[from] fieldinfo::Error),
+    Field(#[from] field::Error),
     /// BitRange error
     #[error("`BitRange error: {0}")]
     BitRange(#[from] bitrange::Error),
@@ -226,3 +226,27 @@ impl<T> EmptyToNone for Option<Vec<T>> {
         self.and_then(|v| if v.is_empty() { None } else { Some(v) })
     }
 }
+
+/// Get SVD element name
+pub trait Name {
+    /// Get name
+    fn name(&self) -> &str;
+}
+
+impl<T> Name for &T
+where
+    T: Name,
+{
+    fn name(&self) -> &str {
+        T::name(*self)
+    }
+}
+
+impl<T> Name for &mut T
+where
+    T: Name,
+{
+    fn name(&self) -> &str {
+        T::name(*self)
+    }
+}
